@@ -1,5 +1,5 @@
 # Standard library imports
-from typing import Dict, Optional, Union, Type
+from typing import Dict, Optional, Union, Type, List
 
 # Third-party imports
 import numpy as np
@@ -23,9 +23,12 @@ from policyengine_uk.utils.scenario import Scenario
 from policyengine_uk.data.economic_assumptions import (
     extend_single_year_dataset,
 )
+from policyengine_uk.utils.dependencies import get_variable_dependencies
 
 from .tax_benefit_system import CountryTaxBenefitSystem
 
+from microdf import MicroDataFrame
+
 
 class Simulation(CoreSimulation):
     """UK-specific simulation class for calculating tax and benefit outcomes.
@@ -36,6 +39,10 @@ class Simulation(CoreSimulation):
 
     default_input_period: int = 2025
     default_calculation_period: int = 2025
+    baseline: "Simulation"
+
+    calculated_periods: List[str] = []
+    _variable_dependencies: Dict[str, List[str]] = None
 
     def __init__(
         self,
@@ -103,10 +110,6 @@ class Simulation(CoreSimulation):
         else:
             raise ValueError(f"Unsupported dataset type: {dataset.__class__}")
 
-        # Handle behavioral responses for earnings and capital gains
-        self.move_values("employment_income", "employment_income_before_lsr")
-        self.move_values("capital_gains", "capital_gains_before_response")
-
         self.input_variables = self.get_known_variables()
 
         # Universal Credit reform (July 2025). Needs closer integration in the baseline,
@@ -117,10 +120,29 @@ class Simulation(CoreSimulation):
 
         # Apply structural modifiers
 
+        self.tax_benefit_system.reset_parameter_caches()
+
         if scenario is not None:
             if scenario.simulation_modifier is not None:
                 scenario.simulation_modifier(self)
 
+        if scenario is not None:
+            self.baseline = Simulation(
+                scenario=None,
+                situation=situation,
+                dataset=dataset,
+                trace=trace,
+            )
+        else:
+            self.baseline = self.clone()
+
+        self.calculated_periods = []
+
+    def reset_calculations(self):
+        for variable in self.tax_benefit_system.variables:
+            if variable not in self.input_variables:
+                self.delete_arrays(variable)
+
     def get_known_variables(self):
         variables = []
         for variable in self.tax_benefit_system.variables:
@@ -165,6 +187,10 @@ class Simulation(CoreSimulation):
         builder.build_from_dict(self.tax_benefit_system, situation, self)
         self.has_axes = builder.has_axes
 
+        self.dataset = UKSingleYearDataset.from_simulation(
+            self, fiscal_year=self.default_input_period
+        )
+
     def build_from_url(self, url: str) -> None:
         """Build simulation from a HuggingFace dataset URL.
 
@@ -197,14 +223,11 @@ class Simulation(CoreSimulation):
         # Determine dataset type and build accordingly
         if UKMultiYearDataset.validate_file_path(dataset, False):
             self.build_from_multi_year_dataset(UKMultiYearDataset(dataset))
-            self.dataset = dataset
         elif UKSingleYearDataset.validate_file_path(dataset, False):
             self.build_from_single_year_dataset(UKSingleYearDataset(dataset))
-            self.dataset = dataset
         else:
             dataset = Dataset.from_file(dataset, self.default_input_period)
             self.build_from_dataset(dataset)
-            self.dataset = dataset
 
     def build_from_dataframe(self, df: pd.DataFrame) -> None:
         """Build simulation from a pandas DataFrame.
@@ -405,13 +428,43 @@ class Simulation(CoreSimulation):
                 simulation.set_input(variable_target, known_period, array)
                 holder.delete_arrays(known_period)
 
+    def calculate_all(self, year: int) -> None:
+        person, benunit, household = (
+            pd.DataFrame(),
+            pd.DataFrame(),
+            pd.DataFrame(),
+        )
+        period = str(year)
+        for variable in self.tax_benefit_system.variables.values():
+            entity = variable.entity.key
+            result = self.calculate(variable.name, period).values
+            if entity == "person":
+                person[variable.name] = result
+            elif entity == "benunit":
+                benunit[variable.name] = result
+            elif entity == "household":
+                household[variable.name] = result
+
+        person = MicroDataFrame(person, weights="person_weight")
+        benunit = MicroDataFrame(benunit, weights="benunit_weight")
+        household = MicroDataFrame(household, weights="household_weight")
+
+        return UKSingleYearDataset(
+            person=person,
+            benunit=benunit,
+            household=household,
+            fiscal_year=year,
+        )
+
     def calculate(
         self,
-        variable_name: str,
+        variable_name: str = None,
         period: str = None,
         map_to: str = None,
         decode_enums: bool = False,
     ):
+        if variable_name is None:
+            return self.calculate_all()
         tracer: SimpleTracer = self.tracer
         if len(tracer.stack) == 0:
             # Only decode enums to string values when we're not within
@@ -426,3 +479,127 @@ class Simulation(CoreSimulation):
         return super().calculate(
             variable_name, period, map_to=map_to, decode_enums=decode_enums
         )
+
+    def apply_dynamics(self, year: int):
+        """Apply dynamics to the simulation for a specific year.
+
+        Args:
+            year: Year to apply dynamics for
+        """
+        # Apply OBR labour supply dynamics if enabled
+        from policyengine_uk.dynamics.labour_supply import (
+            apply_labour_supply_responses,
+        )
+
+        return apply_labour_supply_responses(self, year=str(year))
+
+    def get_variable_dependencies(self, variable_name: str, depth: int = 1):
+        if self._variable_dependencies is None:
+            self._variable_dependencies = {}
+            self._example_simulation = Simulation(
+                situation={
+                    "age": 30,
+                },
+                trace=True,
+            )
+            self._example_simulation.calculate("hbai_household_net_income")
+            for (
+                variable
+            ) in self._example_simulation.tax_benefit_system.variables:
+                try:
+                    dependencies = get_variable_dependencies(
+                        variable, self._example_simulation
+                    )
+                    self._variable_dependencies[variable] = dependencies
+                except Exception as e:
+                    self._variable_dependencies[variable] = []
+
+        if variable_name not in self._variable_dependencies:
+            raise ValueError(
+                f"Variable '{variable_name}' not found in simulation dependencies."
+            )
+        dependencies = self._variable_dependencies[variable_name]
+
+        if depth <= 1:
+            return dependencies
+
+        all_dependencies = set(dependencies)
+        for dep in dependencies:
+            try:
+                sub_dependencies = self.get_variable_dependencies(
+                    dep, depth - 1
+                )
+                all_dependencies.update(sub_dependencies)
+            except ValueError:
+                continue
+        return list(all_dependencies)
+
+    def compare(
+        self,
+        other: "Simulation",
+        variables: list[str] = None,
+        period: str = None,
+        change_only: bool = False,
+    ):
+        """Compare two simulations for a specific variable list.
+
+        Args:
+            other: Another Simulation instance to compare against
+            variables: List of variable names to compare. If None, compares all variables.
+
+        Returns:
+            DataFrame with comparison results
+        """
+
+        if variables is None:
+            variables = self.input_variables + [
+                "hbai_household_net_income",
+            ]
+
+        df_self = self.calculate_dataframe(variables, period=period)
+        df_other = other.calculate_dataframe(variables, period=period)
+
+        df_combined = pd.concat(
+            [df_self, df_other], axis=1, keys=["self", "other"]
+        )
+        # Reset columns, then order by variable name first then self/other
+        df_combined.columns = [
+            f"{var}_{entity}" for entity, var in df_combined.columns
+        ]
+        # Add change where applicable (numeric only)
+        for variable in variables:
+            self_col = f"{variable}_self"
+            other_col = f"{variable}_other"
+            try:
+                df_combined[f"{variable}_change"] = (
+                    df_combined[other_col] - df_combined[self_col]
+                )
+            except:
+                # For other data types, use XOR
+                df_combined[f"{variable}_change"] = (
+                    df_combined[other_col] != df_combined[self_col]
+                )
+        columns = []
+
+        for variable in variables:
+            columns.extend(
+                [
+                    f"{variable}_self",
+                    f"{variable}_other",
+                    f"{variable}_change",
+                ]
+            )
+
+        if change_only:
+            variables_to_include = []
+            for variable in variables:
+                if df_combined[f"{variable}_change"].mean() > 0:
+                    variables_to_include.extend(
+                        [
+                            f"{variable}_self",
+                            f"{variable}_other",
+                            f"{variable}_change",
+                        ]
+                    )
+            columns = variables_to_include
+        return df_combined[columns]
